/*###########################################################
  # Makefile source for Unix tools. Configure makefile and  #
  # then run "make" to create "makefile.unx and compile     #
  ###########################################################
*/

LIBDIR=$(LOCAL)/lib
BINDIR=$(LOCAL)/bin
INCDIR=$(LOCAL)/include
OBJ=o
#ifdef USE_GNU_CC
CC=gcc
#endif

#ifdef MAKE_DEBUG_VERSION
DEBUG=-g
#else
DEBUG=-O -DNDEBUG
#endif

/* The following includes Linux 2.0.18 */
#ifdef MLSYSV

#ifdef ELF_SHARED_LIBRARIES
PIC=-fPIC
#endif

OS=-DML_SYSV
#undef NO_NDIR
#undef NO_STRSTR
#undef NO_GETOPT
CFLAGS=$(OS) $(PIC) $(DEBUG) -I $(INCDIR) -DUSG

#endif /* ML_SYSV */

#ifdef MLXENIX
OS=-DML_XENIX
#define NO_NDIR 1
#define NO_STRSTR 1
#define NO_GETOPT 1
#ifdef USE_GNU_CC
CC=gcc
CFLAGS=$(OS) $(DEBUG) -I $(INCDIR) -DUSG

#else /* Native compiler */
CFLAGS=$(OS) $(DEBUG) -I $(INCDIR) -DML_XENIX -DUSG
#endif /* !USE_GNU_CC */

#endif /* ML_XENIX */

#ifdef MLTURBOC
OS=-DML_TURBOC
CC=tcc
OBJ=obj
#define NO_NDIR 1
#define NO_PIPE 1
#define NO_GETOPT 1
#define NO_UTIMES 1
#define NO_ALLOCA 1
#define NEEDS_FIXNAME 1
#ifdef MAKE_DEBUG_VERSION
DEBUG=-v -y -N  /* No optimise; add debugging */
#else
DEBUG=-G -O -N- /* Optimise for speed */
#endif /* MAKE_DEBUG_VERSION */

CFLAGS=-ml $(OS) $(DEBUG) -I$(INCDIR) -DML_TURBOC

#endif /* ML_TURBOC */

#ifdef MLSUNOS
OS=-DML_SUNOS
#ifdef USE_GNU_CC
CC=gcc
/* CFLAGS=-nostdinc -O -I . -I- -I /usr/5include -I /usr/include -I$(INCDIR) -DML_SUNOS */
CFLAGS= -O -I . -I$(INCDIR) -DML_SUNOS

#else /* Native compiler */
CC=/usr/5bin/cc

#define NO_ALLOCA

#ifdef SUN_SHARED_LIBRARIES
PIC=-pic
#endif

#ifdef MAKE_DEBUG_VERSION
CFLAGS=$(OS) -g -I$(INCDIR) -DML_SUNOS
#else
CFLAGS=$(OS) -O $(PIC) -I$(INCDIR) -DML_SUNOS
#endif /* MAKE_DEBUG_VERSION */

#endif /* !USE_GNU_CC */

#endif /* ML_SUNOS */

#ifdef USE_GNU_CC
#undef NO_ALLOCA
#endif

#ifdef NO_ALLOCA
ALLOCA=alloca.$(OBJ)
#endif

#ifdef NO_STRSTR
STRSTROBJ=strstr.$(OBJ)
#endif

#ifdef NO_NDIR
NDIROBJ=ndir.$(OBJ) 
NDIRINC=ndir.h
#endif

#ifdef NO_PIPE
PIPEOBJ=popen.$(OBJ)
#endif

#ifdef NO_GETOPT
GETOPTOBJ=getoptx.$(OBJ)
GETOPTINC=getopt.h
#endif

#ifdef NO_UTIMES
UTIMESOBJ=utimes.$(OBJ)
#endif

#ifdef NEEDS_FIXNAME
FIXNAME=fixname.$(OBJ)
#endif

OTHEROBJ=$(ALLOCA) $(STRSTROBJ) $(NDIROBJ) $(PIPEOBJ) $(GETOPTOBJ) $(UTIMESOBJ) $(FIXNAME)

LIBS=	fname.$(OBJ)   regex.$(OBJ)    re_fail.$(OBJ) fqn.$(OBJ) \
        field.$(OBJ)   addcomma.$(OBJ) undo_esc.$(OBJ) \
	glob.$(OBJ)    backup.$(OBJ)   pathopen.$(OBJ) \
	string.$(OBJ)  array.$(OBJ)    $(OTHEROBJ)

HFILES=mylib.h $(GETOPTINC) $(NDIRINC)

#ifdef MLTURBOC
LIB=mylibl.lib
$(LIB): $(LIBS)
	/* subex is 'like' sed ... part of my MSDOS toolkit */
	tlib $@ @mylib.tlb
#else /* !ML_TURBOC */

#ifdef SUN_SHARED_LIBRARIES
LIB=libmylib.so.1.00
$(LIB): $(LIBS)
	/usr/bin/ld -assert pure-text -o $@ $(LIBS)

#elif ELF_SHARED_LIBRARIES
#define Major(x) x ## MAJOR
#define Minor(x) x ## MINOR
LIB=libmylib.so
LIB1=Major($(LIB))
LIB2=Minor(Major($(LIB)))

$(LIB): $(LIBS)
	gcc -shared -Wl,-soname,libmylib.so.1 -o $@ $(LIBS)
	ln -s libmylib.so.1.0 libmylib.so.1
	ln -s libmylib.so.1 libmylib.so
#else
LIB=libmylib.a
$(LIB): $(LIBS)
	ar rcv $@ $?
	ranlib $@
#endif /* ML_SUNOS && MAKE_DEBUG_VERSION */
#endif /* !ML_TURBOC */

$(LIBDIR):
	mkdir $(LIBDIR)

$(INCDIR):
	mkdir $(INCDIR)

all: $(LIB)

	
#if ELF_SHARED_LIBRARIES
install: all
	@if [ ! -d $(LIBDIR) ] ; then echo $(LIBDIR) is not a directory ; exit 1; fi
	@if [ ! -d $(INCDIR) ] ; then echo $(INCDIR) is not a directory ; exit 1; fi
	rm -f $(LIBDIR)/libmylib.*
	cp $(LIB) $(LIBDIR)
	(cd $(LIBDIR); ln -s $(LIB) libmylib.so.1; ln -s libmylib.so.1 libmylib.so)
	cp $(HFILES) $(INCDIR)

#else
install: all
	@if [ ! -d $(LIBDIR) ] ; then echo $(LIBDIR) is not a directory ; exit 1; fi
	@if [ ! -d $(INCDIR) ] ; then echo $(INCDIR) is not a directory ; exit 1; fi
	rm -f $(LIBDIR)/libmylib.*
	cp $(LIB) $(LIBDIR)
	cp $(HFILES) $(INCDIR)
#endif

clean:
	rm -f $(LIB) $(LIBS) *~ *.bak

