###########################################################
# Makefile source for Unix tools. Configure makefile and  #
# then run "make"                                         #
###########################################################

LIBDIR=_LOCAL_/lib
BINDIR=_LOCAL_/bin
INCDIR=_LOCAL_/include
OBJ=o
DELCMD=rm
define(_REVISION_,_MAJOR_._MINOR_)

ifdef(`_DEBUG_', DEBUG=-g, DEBUG=-O -DNDEBUG)

ifelse(_OS_, MLSYSV,
# makefile.use for SYSV including Linux 1.?, 2.0.18
OS=-DML_SYSV
ifdef(`_DEBUG_', DEBUG=-g, DEBUG=-O -DNDEBUG)
ifdef(`_SHARED_LIBS_', PIC=-fPIC)

CFLAGS=$(OS) $(PIC) $(DEBUG) -I $(INCDIR) -DUSG

# ML_SYSV
)

ifelse(_OS_, MLTURBOC,
# makefile for TurboC
OS=-DML_TURBOC
CC=tcc
OBJ=obj
DELCMD=DEL

ALLOCA=alloca.$(OBJ)
NDIROBJ=ndir.$(OBJ)
NDIRINC=ndir.h
PIPEOBJ=popen.$(OBJ)
GETOPTOBJ=getoptx.$(OBJ)
GETOPTINC=getopt.h
UTIMESOBJ=utimes.$(OBJ)
FIXNAME=fixname.$(OBJ)

ifdef(`_DEBUG_', 
# No optimise; add debugging
DEBUG=-v -y -N
,
# Optimise for speed */
DEBUG=-G -O -N- -DNDEBUG
)

ifdef(`_SHARED_', aaarrrrgggghhhhh - no shared libraries in TurboC!)
CFLAGS=-ml $(OS) $(DEBUG) -I$(INCDIR)

# ML_TURBOC
)

ifelse(_OS_, MLSUNOS,
# makefile for SunOS, (Solaris??)
OS=-DML_SUNOS

ifdef(`_USE_GNU_CC_',
CC=gcc
/* CFLAGS=-nostdinc -O -DNDEBUG -I . -I- -I /usr/5include -I /usr/include -I$(INCDIR) -DML_SUNOS */
CFLAGS= -O -DNDEBUG -I . -I$(INCDIR) -DML_SUNOS

, # else /* Native compiler */
CC=/usr/5bin/cc

define(_NO_ALLOCA_,1)

ifdef(`_SHARED_LIBS_',PIC=-pic)

ifdef(`_DEBUG_',
CFLAGS=$(OS) -g -I$(INCDIR) -DML_SUNOS,
CFLAGS=$(OS) -O -DNDEBUG $(PIC) -I$(INCDIR) -DML_SUNOS)

))

ifdef(`_USE_GNU_CC_', define(_NO_ALLOCA_, 0))


OTHEROBJ=$(ALLOCA) $(STRSTROBJ) $(NDIROBJ) $(PIPEOBJ) $(GETOPTOBJ) $(UTIMESOBJ) $(FIXNAME)

LIBS=	fname.$(OBJ)   regex.$(OBJ)    re_fail.$(OBJ) fqn.$(OBJ) \
        field.$(OBJ)   addcomma.$(OBJ) undo_esc.$(OBJ) \
	glob.$(OBJ)    backup.$(OBJ)   pathopen.$(OBJ) \
	string.$(OBJ)  array.$(OBJ)    $(OTHEROBJ)

HFILES=mylib.h $(GETOPTINC) $(NDIRINC)

ifelse(_OS_, MLTURBOC,
LIB=mylibl.lib
$(LIB): $(LIBS)
	/* subex is 'like' sed ... part of my MSDOS toolkit */
	tlib $@ @mylib.tlb
# ML_TURBOC
)

ifelse(_OS_, MLSUNOS,

ifdef(`_SHARED_LIBS_',
LIB=libmylib.so._MAJOR_._MINOR_
$(LIB): $(LIBS)
	/usr/bin/ld -assert pure-text -o $@ $(LIBS)
))

ifelse(_OS_, MLSYSV,

ifdef(`_SHARED_LIBS_',
LIB=libmylib.so
LIB1=$(LIB)._MAJOR_
LIB2=$(LIB)._MAJOR_._MINOR_

$(LIB): $(LIBS)
	gcc ``-shared -Wl,-soname,$(LIB1)'' -o $(LIB2) $(LIBS)
	ln -s $(LIB2) $(LIB1)
	ln -s $(LIB1) $(LIB)

, #else !SHARED_LIBS
LIB=libmylib.a
$(LIB): $(LIBS)
	ar rcv $@ $?
	ranlib $@
))

$(LIBDIR):
	mkdir $(LIBDIR)

$(INCDIR):
	mkdir $(INCDIR)

all: $(LIB)

install: all
	@if [ ! -d $(LIBDIR) ] ; then echo $(LIBDIR) is not a directory ; exit 1; fi
	@if [ ! -d $(INCDIR) ] ; then echo $(INCDIR) is not a directory ; exit 1; fi
	cp $(HFILES) $(INCDIR)
	rm -f $(LIBDIR)/libmylib.*
	ifdef(`_SHARED_LIBS_', cp $(LIB2) $(LIBDIR);(cd $(LIBDIR); ln -s $(LIB2) $(LIB1); ln -s $(LIB1) $(LIB)), cp $(LIB) $(LIBDIR))

clean:
	rm -f $(LIB) $(LIB1) $(LIB2) *.$(OBJ) *~

ifelse(_OS_, MLSYSV,`
# Assumes that ../dcalc-_REVISION_ exists and has links to the appropriate files here
# Probably needs to run as root
rpm:
	(cd ..;	tar cvzf /usr/src/redhat/SOURCES/mylib-_REVISION_-_RELEASE_.tgz mylib-_REVISION_ `)'
	rpm -ba mylib-_REVISION_-_RELEASE_.spec
	rm -rf /usr/src/redhat/BUILD/mylib-_REVISION_
	rm -rf /usr/src/redhat/SOURCES/mylib-_REVISION_
	echo "RPMs are"
	echo /usr/src/redhat/SRPMS/mylib-_REVISION_*
	echo /usr/src/redhat/RPMS/i386/mylib-_REVISION_*

tar:
	(cd ..; tar cvzf mylib-_REVISION_-_RELEASE_.tgz mylib-_REVISION_ )

')